function classifier = ...
    train_global(data_dir, pos_clip_list, neg_clip_list, num_synth, num_neg, training_params, varargin)
% Learn the classification function for a sound object
%
% INPUT:
% data_dir: the directory where the training audio wav files are stored
% pos_clip_list: list of short clips that contain the sound of interest
% neg_clip_list: list of (possibly longer) audio files that do not contain the sound of interest
% num_synth: the number of synthetic examples to produce for each positive example
% num_neg: the number of negative samples to use
% training_params.{fs, band_boundaries, window_size, overlapping_windows}: training parameters
% varargin: {number of previous stages, classifiers for previous stages} (optional)
% 
% OUTPUT
% classifier: the boosted decision tree classifier used to locate sound objects
%
% Features:
%   band_mean_t: mean power of each band over time (1-16) - how powerful is each band
%   band_std_t: std power of each band over time (17-32) - how much does each band vary
%   mpb: most powerful band of 4 (33), most powerful band of 16 (34) -
%           which is the primary band (at coarse and fine resolution)            
%   mpb_std_t: std of most powerful band of 16 (35) - how consistent is mpb
%   band_mean_b: mean band over time, bands 
%   band_width_50: width of smallest set of bands containing 50% of power (36) -
%           how concentrated is the frequency power
%   nsb_mean_t: mean of number of bands with greater than 25% power (37)
%   nsb_std_t: std of number of bands with greater than 25% power (38)
%   power_std: std of total power over time (39)
%   power_peaks: number of power peaks (40)
%   periodicity: regularity of power peaks (41)
%   power_range: max_power - min_power (42)
%   mean_power: mean of power (43)
%   band_mean_t_4: coarse band means (44-47)
%   band_std_t_4: coarse band stds (48-51)
%   52 - 138: time-dependent frequency information at different resolutions
      
                 
num_prev_stages = 0;
num_var_args = length(varargin);   
if num_var_args > 0 
    num_prev_stages = varargin{1}(1);        
    prev_classifiers = varargin{2}(:);
end
fraction_of_pos_list_to_use = 1.0;        

num_bands = length(training_params.band_boundaries);
num_features = 138;

neg_clips = read_wav_files(data_dir, neg_clip_list);
pos_clips = read_wav_files(data_dir, pos_clip_list);

pos_clips = pos_clips(1:round(length(pos_clips)*fraction_of_pos_list_to_use));
%num_synth = max(round(num_neg/length(pos_clips)), 1);


% data(num_times, num_samples) 
% by convention, all positive examples are first
% samples have no border

% get the length of a sound object based on the median length of
% positive clips
med_length = floor(get_median_clip_length(pos_clips));
med_length = min([med_length 2.5*8000]);
% make length an integer multiple of 0.5 seconds
med_length = round(med_length / 4000) * 4000;
classifier.data_length = med_length;

disp(['median length = ', num2str(med_length/8000)]);
if num_prev_stages > 0
    [data, labels, confidences] = ...
        create_uniform_length_clips(pos_clips, neg_clips, num_synth, num_neg, med_length, ...
                                    num_prev_stages, prev_classifiers);            
else
    [data, labels, confidences] = ...
        create_uniform_length_clips(pos_clips, neg_clips, num_synth, num_neg, med_length);    
end        

num_samples = size(data, 2);    
num_pos = sum(labels == 1);
num_neg = num_samples - num_pos;

disp('computing features')
features = zeros(num_features, num_samples);
for s = 1:num_samples
    [features(1:num_features, s), classifier.window_length] = ...
        extract_global_features_window(data(:, s), training_params);
end        

disp('computing classifier parameters');
classifier.wcs = compute_decision_tree(features, labels, 5);
   
classifier.fft_params = training_params;


